#include <LiquidCrystal_I2C.h>
#include <OneWire.h>
#include <EEPROM.h>


#define BACKLIGHT_PIN     13

LiquidCrystal_I2C lcd(0x27,2,1,0,4,5,6,7,3,POSITIVE);  // Set the LCD I2C address
OneWire  OneWireBus(8);  // on pin 8 (a 4.7K resistor is necessary)

byte OneWireBuffer;
byte ROM_ID[8];

byte v_blink;
byte v_lower;

byte ROM_ID_T1[8];
byte ROM_ID_T2[8];

byte ROM_ID_RTC[8];


byte ow_mx[10];
byte ow_rmx[10];
byte i2c_eeprom[8];
byte i;
byte ow_T1_t0;
byte ow_T1_t1;
byte ow_T2_t0;
byte ow_T2_t1;

const int set_T1 = 12;
const int set_T2 = 11;
const int set_T3 = 10;
const int set_T4 =  9;

int set_T1_val;
int set_T2_val;
int set_T3_val;
int set_T4_val;
int set_Tx;


byte analogPin = A0;
int analogVal = 0;
int set_hrs;

byte nss_done;
byte T1_fit;
byte T2_fit;
byte T3_fit;
byte T4_fit;

int Fnc1State;
int Fnc2State;
int S3State;

byte temp_whole;
byte temp_dec;
byte temp_sign;

byte e_adr_L;
byte e_adr_H;

byte v_cnt;
byte demo_on;

unsigned long rtc_counter;
unsigned long rtc_days;
byte rtc_hrs;
byte rtc_min;
byte rtc_sec;

byte ee_address;
byte v_hrs;
byte v_ee_hrs;
byte v_ee_day;
byte lv_st1;
byte lv_cnt1;

byte lv_dcm;      //Display Change Mode

byte rtc_byte0;
byte rtc_byte1;
byte rtc_byte2;

byte v_disp_temp;

byte v_c4;
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void setup()
{
  // Switch on the backlight
  pinMode ( BACKLIGHT_PIN, OUTPUT );
  digitalWrite ( BACKLIGHT_PIN, HIGH );
  
  lcd.setBacklight(HIGH);
  
  lcd.begin(20,4);               // initialize the lcd 

  lcd.setCursor ( 0, 0 );
  lcd.print("24-hour Thermometer");
  lcd.setCursor ( 0, 1 );
  lcd.print ("Arduino v1.12  ");
  lcd.setCursor ( 0, 2 );
  lcd.print ("17.05.2020     ");
  lcd.setCursor ( 0, 3 );
  lcd.print ("Dual Sensor Support");
  delay ( 4000 );
  lcd.clear();

  pinMode(set_T1, INPUT);
  digitalWrite(set_T1, HIGH);  
  
  pinMode(set_T2, INPUT);
  digitalWrite(set_T2, HIGH);
  
  pinMode(set_T3, INPUT);
  digitalWrite(set_T3, HIGH);
  
  pinMode(set_T4, INPUT);
  digitalWrite(set_T4, HIGH);

  v_blink = 0;

  init_OneWire_Chips();

  lv_st1 = 1;
  lv_cnt1 = 0;
  lv_dcm = 0;

  v_disp_temp = 1;
  v_c4 = 0;
}
//--------------------------------------------------------
//--------------------------------------------------------
// ----- MAIN LOOP ---------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void loop()
{
   init_OneWire_Chips();
// ------------------------------
   rtc_read_data();
   rtc_calc_time();
   lcd.setCursor (3,0);
   if (ROM_ID_RTC[0] == 0x00) lcd.print("--:--:--");
   else write_time();
// ------------------------------
   temp_conv_spu();
   read_scp_T1();
   read_scp_T2();

   set_T2_val = digitalRead(set_T2);
   if (set_T2_val == 1) v_disp_temp = 1;    //default: input = 1 -> Sensor A
   else v_disp_temp = 2;
   
   if (v_disp_temp == 1)
     {
       ow_mx[0] = ow_T1_t0;
       ow_mx[1] = ow_T1_t1;
     }
   else
     {
       ow_mx[0] = ow_T2_t0;
       ow_mx[1] = ow_T2_t1;
     }
   lcd.setCursor (12,0);
   if (v_disp_temp == 1)
      {
        if (ROM_ID_T1[0] != 0x00)
           {
             temp_calc_qt(ROM_ID_T1[0]);
             write_temp(1);
           }
        else
           {
             lcd.print("  --- ");
             lcd.print((char)223);
             lcd.print("C");
           }
      }
   else
      {
        if (ROM_ID_T2[0] != 0x00)
          {
            temp_calc_qt(ROM_ID_T2[0]);
            write_temp(1);
          }
        else
           {
             lcd.print("  --- ");
             lcd.print((char)223);
             lcd.print("C");
           }
      }
   
// ------------------------------  
   lcd.setCursor (0,0);
   switch(lv_dcm)
      {
        case 0:
          lcd.print("L");
          hist_curr_last();
          break;
        case 1:
          lcd.print("1");
          lv_st1 = 1;
          break;
        case 2:
          lcd.print("2");
          lv_st1 = 7;
          break;
        case 3:
          lcd.print("3");
          lv_st1 = 13;
          break;
        case 4:
          lcd.print("4");
          lv_st1 = 19;
          break;
      }

   if (v_disp_temp == 1) lcd.print("A");
   else lcd.print("B");
   
   write_history(lv_st1);
   
   if (ROM_ID_RTC[0] != 0x00) store_temp_T1();
   if (ROM_ID_RTC[0] != 0x00) store_temp_T2();

   set_T1_val = 1;
   set_T1_val = digitalRead(set_T1);
   if (set_T1_val == 0) reset_cnt_ee();

   analogVal = analogRead(analogPin);
   lv_dcm = 4;
   if (analogVal < 800) lv_dcm = 3;
   if (analogVal < 600) lv_dcm = 2;
   if (analogVal < 400) lv_dcm = 1;
   if (analogVal < 200) lv_dcm = 0;

   v_c4 = v_c4 + 1;
   if (v_c4 == 10)
     {
      v_c4 = 0;
      v_disp_temp = v_disp_temp + 1;
      if (v_disp_temp == 3) v_disp_temp = 1;
     }
}
//--------------------------------------------------------
//--------------------------------------------------------
void hist_curr_last(void)
{
   if (rtc_hrs > 4) lv_st1 = rtc_hrs - 5;
   else lv_st1 = 25 - (6 - rtc_hrs);
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void hist_change(void)
{
   lv_cnt1 = lv_cnt1 + 1;
   if (lv_cnt1 == 20)
     {
        lv_cnt1 = 0;
        lv_st1 = lv_st1 + 6;
        if (lv_st1 == 25) lv_st1 = 1;
     }  
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void temp_conv_spu(void)
{
   OneWireBus.reset();
   OneWireBus.write(0xCC);            //Skip ROM
   OneWireBus.write(0x44,1);          //Convert Temperature

   delay(375);
   OneWireBus.reset();
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void write_dec2(byte kbd_buffer)
{
  if(kbd_buffer<10) lcd.print("0");
  lcd.print(kbd_buffer);
}
//--------------------------------------------------------
//--------------------------------------------------------
void write_hex(byte kbd_buffer)
{
  if(kbd_buffer<16) lcd.print("0");
  lcd.print(kbd_buffer, HEX);
}
//--------------------------------------------------------
//--------------------------------------------------------
void ow_read_rom_ID(void)
{
   OneWireBus.reset();
   OneWireBus.write(0x33);
   ROM_ID[0] = OneWireBus.read();
   ROM_ID[1] = OneWireBus.read();
   ROM_ID[2] = OneWireBus.read();
   ROM_ID[3] = OneWireBus.read();
   ROM_ID[4] = OneWireBus.read();
   ROM_ID[5] = OneWireBus.read();
   ROM_ID[6] = OneWireBus.read();
   ROM_ID[7] = OneWireBus.read();
   OneWireBus.reset();
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void ds18b20_set_11bit(byte ROM_ID_scp[8])
{
   OneWireBus.reset();
   temp_match_rom(ROM_ID_scp);
   OneWireBus.write(0x4E);           //Write Scratchpad
   OneWireBus.write(0x00);
   OneWireBus.write(0x11);
   OneWireBus.write(0b01011111);    //set 11-Bit Resolution

   OneWireBus.reset();
   temp_match_rom(ROM_ID_scp);
   OneWireBus.write(0x48, 1);       //Copy Scratchpad
   delay(100);
   OneWireBus.reset();
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void read_scp_T1()
{
   OneWireBus.reset();
   temp_match_rom(ROM_ID_T1);
   
   OneWireBus.write(0xBE);           //Read Scratchpad
   ow_mx[0] = OneWireBus.read();
   ow_mx[1] = OneWireBus.read();
   OneWireBus.reset();

   ow_T1_t0 = ow_mx[0];
   ow_T1_t1 = ow_mx[1];
}
//--------------------------------------------------------
//--------------------------------------------------------
void read_scp_T2()
{
   OneWireBus.reset();
   temp_match_rom(ROM_ID_T2);
   
   OneWireBus.write(0xBE);           //Read Scratchpad
   ow_mx[0] = OneWireBus.read();
   ow_mx[1] = OneWireBus.read();
   OneWireBus.reset();

   ow_T2_t0 = ow_mx[0];
   ow_T2_t1 = ow_mx[1];
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void write_history(byte v_start)
{
  v_hrs = v_start;
  lcd.setCursor(0, 1);
  read_single_record();
  write_single_rec();

  inc_hrs();
  lcd.setCursor(0, 2);
  read_single_record();
  write_single_rec();

  inc_hrs();
  lcd.setCursor(0, 3);
  read_single_record();
  write_single_rec();

  inc_hrs();
  lcd.setCursor(11, 1);
  read_single_record();
  write_single_rec();

  inc_hrs();
  lcd.setCursor(11, 2);
  read_single_record();
  write_single_rec();

  inc_hrs();
  lcd.setCursor(11, 3);
  read_single_record();
  write_single_rec();
}
//--------------------------------------------------------
//--------------------------------------------------------
void inc_hrs()
{
  v_hrs = v_hrs + 1;
  if (v_hrs == 24) v_hrs = 0;
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void write_single_rec(void)
{
  if (v_ee_day > 1 || v_ee_hrs != v_hrs)
    {
      write_dec2(v_hrs);
      lcd.print(":  --.-");
    }
  else
    {
      write_dec2(v_ee_hrs);
      lcd.print(":");
      temp_calc_qt(ROM_ID_T1[0]);
      write_temp(0);      
    }
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void write_temp(byte v_stc)
{
  if (temp_whole > 99 || temp_dec > 9 || temp_dec < 0)
     {
       lcd.print(" ---,-");
       lcd.print((char)223);
       lcd.print("C");
     }
  else
    {
      lcd.print(" ");
      if (temp_sign == 0) lcd.print("+");
      else lcd.print("-");
      if (temp_whole < 10) lcd.print(" ");
      lcd.print(temp_whole, DEC);
      lcd.print(",");
      lcd.print(temp_dec, DEC);
      if (v_stc == 1)
        {
          lcd.print((char)223);
          lcd.print("C");
        }
    }
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------- THERMOMETERS ---------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void temp_calc_qt(byte ROM_ID_Fam)
{
   if (ROM_ID_Fam == 0x10) temp_calc_S20();
   else temp_calc_B20();
}
//--------------------------------------------------------
//--------------------------------------------------------
void temp_calc(void)
{
   if (ROM_ID[1] == 0x10) temp_calc_S20();
   else temp_calc_B20();
}
//--------------------------------------------------------
//--------------------------------------------------------
void temp_calc_B20(void)  
{
   int v_help;  
   
   temp_sign = bitRead(ow_mx[1],7);
   if (temp_sign == 1) {
      ow_mx[0] = ~ow_mx[0];
      ow_mx[1] = ~ow_mx[1];
                      }
   temp_whole = (ow_mx[0] & B11110000) / 16;
   temp_whole = temp_whole + (ow_mx[1] * 16);

   v_help = ((ow_mx[0] & B00001111) * 625) + 500;
   temp_dec = v_help / 1000;
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void temp_calc_S20(void)
{
   int v_help;  
   
   temp_sign = bitRead(ow_mx[1],7);
   if (temp_sign == 1) {
      ow_mx[1] = ~ow_mx[0];
      ow_mx[2] = ~ow_mx[1];
                      }
   temp_whole = (ow_mx[0] & B11111110) / 2;

   v_help = ow_mx[0] & B00000001;
   temp_dec = v_help * 5;
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void write_bin(byte ow_mx_in)
{
   write_bit(bitRead(ow_mx_in, 0x07));
   write_bit(bitRead(ow_mx_in, 0x06));
   write_bit(bitRead(ow_mx_in, 0x05));
   write_bit(bitRead(ow_mx_in, 0x04));
   write_bit(bitRead(ow_mx_in, 0x03));
   write_bit(bitRead(ow_mx_in, 0x02));
   write_bit(bitRead(ow_mx_in, 0x01));
   write_bit(bitRead(ow_mx_in, 0x00));
}
//--------------------------------------------------------
//--------------------------------------------------------
void write_bit(byte ow_mx_bit_in)
{
   if (ow_mx_bit_in == 1) lcd.print("1");
   else lcd.print("0");
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void temp_match_rom(byte ROM_ID_scp[8])
{
   OneWireBus.write(0x55);           //Match ROM
   OneWireBus.write(ROM_ID_scp[0]);
   OneWireBus.write(ROM_ID_scp[1]);
   OneWireBus.write(ROM_ID_scp[2]);
   OneWireBus.write(ROM_ID_scp[3]);
   OneWireBus.write(ROM_ID_scp[4]);
   OneWireBus.write(ROM_ID_scp[5]);
   OneWireBus.write(ROM_ID_scp[6]);
   OneWireBus.write(ROM_ID_scp[7]);
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void rtc_match_rom()
{
   OneWireBus.write(0x55);           //Match ROM
   OneWireBus.write(ROM_ID_RTC[0]);
   OneWireBus.write(ROM_ID_RTC[1]);
   OneWireBus.write(ROM_ID_RTC[2]);
   OneWireBus.write(ROM_ID_RTC[3]);
   OneWireBus.write(ROM_ID_RTC[4]);
   OneWireBus.write(ROM_ID_RTC[5]);
   OneWireBus.write(ROM_ID_RTC[6]);
   OneWireBus.write(ROM_ID_RTC[7]);
}
//--------------------------------------------------------
//--------------------------------------------------------

//--------------------------------------------------------
//--------------------------------------------------------
void rtc_osc_start(void)
{
   if (ROM_ID_RTC[0] == 0x00) exit;
   
   OneWireBus.reset();              //(1) - OneWire Reset
   rtc_match_rom();
   OneWireBus.write(0x99);          //(3) - Write Clock
   OneWireBus.write(B00001100);     //(4) - Write Device Control Byte
   OneWireBus.reset();              //(5) - OneWire Reset
}
//--------------------------------------------------------
//--------------------------------------------------------
void rtc_read_data(void)
{
   if (ROM_ID_RTC[0] == 0x00) exit;

   OneWireBus.reset();
   rtc_match_rom();
   OneWireBus.write(0x66);           //(3) - Read Clock
   ow_rmx[1] = OneWireBus.read();    //(4) - Read Device Control Byte
   ow_rmx[2] = OneWireBus.read();    //(5.1) - Read Counter - LSB
   ow_rmx[3] = OneWireBus.read();    //(5.2) - Read Counter
   ow_rmx[4] = OneWireBus.read();    //(5.3) - Read Counter
   ow_rmx[5] = OneWireBus.read();    //(5.4) - Read Counter - MSB
   OneWireBus.reset();               //(6) - OneWire Reset
}
//--------------------------------------------------------
//--------------------------------------------------------
void rtc_calc_time(void)
{
   if (ROM_ID_RTC[0] == 0x00) exit;
 
   unsigned long v_n4 = 16777216;
   unsigned long v_n3 = 65536;
   unsigned long v_n2 = 256;
   unsigned long v_n1 = 1;

   rtc_counter = ow_rmx[5] * v_n4 +
                 ow_rmx[4] * v_n3 +
                 ow_rmx[3] * v_n2 +
                 ow_rmx[2] * v_n1;
 
   rtc_days = rtc_counter / 86400;      //DIV
   rtc_counter = rtc_counter % 86400;   //MOD
   rtc_hrs = rtc_counter / 3600;        //DIV
   rtc_counter = rtc_counter % 3600;    //MOD
   rtc_min = rtc_counter / 60;          //DIV
   rtc_sec = rtc_counter % 60;          //MOD

   if (rtc_days > 1) rtc_reset();
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void rtc_clear(void)
{    
   OneWireBus.reset();
   rtc_match_rom(); 
   OneWireBus.write(0x99);          //(3) - Write Clock
   OneWireBus.write(B00001100);     //(4) - Write Device Control Byte
   OneWireBus.write(0x00);          //(5.1) - Write Counter - LSB
   OneWireBus.write(0x00);          //(5.2) - Write Counter
   OneWireBus.write(0x00);          //(5.3) - Write Counter
   OneWireBus.write(0x00);          //(5.4) - Write Counter - MSB
   OneWireBus.reset();              //(6) - OneWire Reset
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void get_RTC_ROM_ID(void)
{
   ROM_ID_RTC[0] = 0x00;
   OneWireBus.reset();
   while (OneWireBus.search(ROM_ID))
     {
        if (ROM_ID[0] == 0x24 || ROM_ID[0] == 0x27)
           {
             for (int i =  0; i <  8; i++) ROM_ID_RTC[i] = ROM_ID[i];
             exit;
           }
     }
   OneWireBus.reset();
}
//--------------------------------------------------------
//--------------------------------------------------------
void write_time(void)
{
   if (ow_rmx[1] == 0xFF)
     {
        lcd.print("--:--:--");
     }
   else
     {
        write_dec2(rtc_hrs);
        lcd.print(":");
        write_dec2(rtc_min);
        lcd.print(":");
        write_dec2(rtc_sec);  
     }
}
//--------------------------------------------------------
//--------------------------------------------------------
void rtc_reset(void)
{
   if (ow_rmx[1] != 0xFF)
     {
       rtc_clear();
       lcd.setCursor (0,0);
       lcd.print("Counter cleared...");
       delay(1000);
       lcd.setCursor (0,0);
       lcd.print("                    ");
     }
}

//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void rtc_set_counter()
{
   OneWireBus.reset();
   rtc_match_rom(); 
   OneWireBus.write(0x99);          //(3) - Write Clock
   OneWireBus.write(B00001100);     //(4) - Write Device Control Byte
   OneWireBus.write(rtc_byte0);     //(5.1) - Write Counter - LSB
   OneWireBus.write(rtc_byte1);     //(5.2) - Write Counter
   OneWireBus.write(rtc_byte2);     //(5.3) - Write Counter
   OneWireBus.write(0x00);          //(5.4) - Write Counter - MSB
   OneWireBus.reset();              //(6) - OneWire Reset
}
//--------------------------------------------------------
//--------------------------------------------------------
void rtc_set_to_hrs()
{
   if (ow_rmx[1] != 0xFF)
     {
        rtc_byte0 = 0x40;
        rtc_byte1 = 0x19;
        rtc_byte2 = 0x01;

        switch(set_hrs)
          {
            case 0:
              rtc_byte0 = 0x00;
              rtc_byte1 = 0x00;
              rtc_byte2 = 0x00;
              break;
            case 1:
              rtc_byte0 = 0x10;
              rtc_byte1 = 0x0E;
              rtc_byte2 = 0x00;
              break;
            case 2:
              rtc_byte0 = 0x20;
              rtc_byte1 = 0x1C;
              rtc_byte2 = 0x00;
              break;
            case 3:
              rtc_byte0 = 0x30;
              rtc_byte1 = 0x2A;
              rtc_byte2 = 0x00;
              break;
            case 4:
              rtc_byte0 = 0x40;
              rtc_byte1 = 0x38;
              rtc_byte2 = 0x00;
              break;
            case 5:
              rtc_byte0 = 0x50;
              rtc_byte1 = 0x46;
              rtc_byte2 = 0x00;
              break;
            case 6:
              rtc_byte0 = 0x60;
              rtc_byte1 = 0x54;
              rtc_byte2 = 0x00;
              break;
            case 7:
              rtc_byte0 = 0x70;
              rtc_byte1 = 0x62;
              rtc_byte2 = 0x00;
              break;
            case 8:
              rtc_byte0 = 0x80;
              rtc_byte1 = 0x70;
              rtc_byte2 = 0x00;
              break;
            case 9:
              rtc_byte0 = 0x90;
              rtc_byte1 = 0x7E;
              rtc_byte2 = 0x00;
              break;
            case 10:
              rtc_byte0 = 0xA0;
              rtc_byte1 = 0x8C;
              rtc_byte2 = 0x00;
              break;
            case 11:
              rtc_byte0 = 0xB0;
              rtc_byte1 = 0x9A;
              rtc_byte2 = 0x00;
              break;
            case 12:
              rtc_byte0 = 0xC0;
              rtc_byte1 = 0xA8;
              rtc_byte2 = 0x00;
              break;
            case 13:
              rtc_byte0 = 0xD0;
              rtc_byte1 = 0xB6;
              rtc_byte2 = 0x00;
              break;
            case 14:
              rtc_byte0 = 0xE0;
              rtc_byte1 = 0xC4;
              rtc_byte2 = 0x00;
              break;
            case 15:
              rtc_byte0 = 0xF0;
              rtc_byte1 = 0xD2;
              rtc_byte2 = 0x00;
              break;
            case 16:
              rtc_byte0 = 0x00;
              rtc_byte1 = 0xE1;
              rtc_byte2 = 0x00;
              break;
            case 17:
              rtc_byte0 = 0x10;
              rtc_byte1 = 0xEF;
              rtc_byte2 = 0x00;
              break;
            case 18:
              rtc_byte0 = 0x20;
              rtc_byte1 = 0xFD;
              rtc_byte2 = 0x00;
              break;
            case 19:
              rtc_byte0 = 0x30;
              rtc_byte1 = 0x0B;
              rtc_byte2 = 0x01;
              break;
            case 20:
              rtc_byte0 = 0x40;
              rtc_byte1 = 0x19;
              rtc_byte2 = 0x01;
              break;
            case 21:
              rtc_byte0 = 0x50;
              rtc_byte1 = 0x27;
              rtc_byte2 = 0x01;
              break;
            case 22:
              rtc_byte0 = 0x60;
              rtc_byte1 = 0x35;
              rtc_byte2 = 0x01;
              break;
            case 23:
              rtc_byte0 = 0x70;
              rtc_byte1 = 0x43;
              rtc_byte2 = 0x01;
              break;
          }
       rtc_set_counter();
       lcd.setCursor (0,0);
       lcd.print("RTC Set to ");
       write_dec2(set_hrs);
       lcd.print(":00:00");
       delay(2000);
       lcd.setCursor (0,0);
       lcd.print("                    ");
     }  
}
//--------------------------------------------------------
//--------------------------------------------------------


//--------------------------------------------------------
//--------------------------------------------------------
void get_temp_ROM_ID_single(void)
{
   ROM_ID_T1[0] = 0x00;
   OneWireBus.reset();
   while (OneWireBus.search(ROM_ID))
     {
        if (ROM_ID[0] == 0x28)
           {
             for (int i =  0; i <  8; i++) ROM_ID_T1[i] = ROM_ID[i];
             exit;
           }
     }
   OneWireBus.reset();
}
//--------------------------------------------------------
//--------------------------------------------------------
void get_temp_ROM_ID(void)
{
   ROM_ID_T1[0] = 0x00;
   ROM_ID_T2[0] = 0x00;
   OneWireBus.reset();
   while (OneWireBus.search(ROM_ID))
     {
        if (ROM_ID[0] == 0x28)
           {
             if (ROM_ID_T1[0] == 0x00)
               for (int i =  0; i <  8; i++) ROM_ID_T1[i] = ROM_ID[i];
             else
               {
                 for (int i =  0; i <  8; i++) ROM_ID_T2[i] = ROM_ID[i];
                 exit;
               }
           }
     }
   OneWireBus.reset();
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void reset_cnt_ee(void)
{
  if (ROM_ID_RTC[0] != 0x00)
    {
      lcd.clear();
      lcd.setCursor(0,3);
      lcd.print("Set clk to: xx:00:00");
  
      lcd.setCursor(0,0);
      lcd.print("Init in progres...");

      init_eeprom();
      lcd.setCursor(0,0);
      lcd.print("Clock  initialized");
      lcd.setCursor(0,1);
      lcd.print("EEPROM initialized");
      while(set_T1_val == 0)
        {
          set_T1_val = digitalRead(set_T1);
          analogVal = analogRead(analogPin);
          set_hrs = ( analogVal * 2 ) / 88;
          lcd.setCursor(12, 3);
          write_dec2(set_hrs);
        }

      lcd.clear();
      rtc_set_to_hrs();
    }
}

//--------------------------------------------------------
//--------------------------------------------------------
void init_eeprom(void)
{
  for (int i = 0; i < 256; i++) EEPROM.write(i, 0xFF);
}
//--------------------------------------------------------
//--------------------------------------------------------
void test_ee_write(void)
{
  EEPROM.write(0x04,0x00);
  EEPROM.write(0x05,0x01);
  EEPROM.write(0x06,0x17);
  EEPROM.write(0x07,0x00);

  EEPROM.write(0x0C,0x00);
  EEPROM.write(0x0D,0x03);
  EEPROM.write(0x0E,0x37);
  EEPROM.write(0x0F,0x01);
}
//--------------------------------------------------------
//--------------------------------------------------------
void read_single_record()
{
  byte ee_addr_local;
  
  ee_address = v_hrs * 4;
  ee_addr_local = ee_address + ( v_disp_temp - 1 ) * 128;
  
  v_ee_day = EEPROM.read(ee_addr_local);
  v_ee_hrs = EEPROM.read(ee_addr_local + 1);
  ow_mx[0] = EEPROM.read(ee_addr_local + 2);
  ow_mx[1] = EEPROM.read(ee_addr_local + 3);
}
//--------------------------------------------------------
//--------------------------------------------------------
void store_temp_T1(void)
{
  byte v_ee_day_tmp;
  byte v_dtt;

  if (ROM_ID_T1[0] != 0x00)
    {
      v_dtt = v_disp_temp;
      v_disp_temp = 1;
      v_ee_day_tmp = rtc_days;
      v_hrs = rtc_hrs;
      read_single_record();
      if (v_ee_day != rtc_days || v_ee_hrs != v_hrs)
        {
          EEPROM.write(ee_address, v_ee_day_tmp);
          EEPROM.write(ee_address + 1, rtc_hrs);
          EEPROM.write(ee_address + 2, ow_T1_t0);
          EEPROM.write(ee_address + 3, ow_T1_t1);
        }
      v_disp_temp = v_dtt;
    }
}
//--------------------------------------------------------
//--------------------------------------------------------
void store_temp_T2(void)
{
  byte v_ee_day_tmp;
  
  byte v_dtt;

  if (ROM_ID_T2[0] != 0x00)
    {
      v_dtt = v_disp_temp;
      v_disp_temp = 2;
      v_ee_day_tmp = rtc_days;
      v_hrs = rtc_hrs;
      read_single_record();
      if (v_ee_day != rtc_days || v_ee_hrs != v_hrs)
        {
          EEPROM.write(ee_address + 128, v_ee_day_tmp);
          EEPROM.write(ee_address + 128 + 1, rtc_hrs);
          EEPROM.write(ee_address + 128 + 2, ow_T2_t0);
          EEPROM.write(ee_address + 128 + 3, ow_T2_t1);
        }
      v_disp_temp = v_dtt;
    }
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void init_OneWire_Chips()
{
  get_RTC_ROM_ID();
  get_temp_ROM_ID();
  
  rtc_osc_start();
  ds18b20_set_11bit(ROM_ID_T1);
  ds18b20_set_11bit(ROM_ID_T2);
}
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------
void eeprom_dump()
{
   byte ee_byte;
   byte ee_adr;
   
   lcd.clear();
   
   while (1)
     {
       ee_adr = 0x1C;
      
       ee_byte = EEPROM.read(0);
       lcd.setCursor(0,0);
       write_hex(ee_byte);
       
       ee_byte = EEPROM.read(ee_adr + 1);
       lcd.print("-");
       write_hex(ee_byte);

       ee_byte = EEPROM.read(ee_adr + 2);
       lcd.print("-");
       write_hex(ee_byte);

       ee_byte = EEPROM.read(ee_adr + 3);
       lcd.print("-");
       write_hex(ee_byte);
     }  
}
//--------------------------------------------------------
//--------------------------------------------------------
